﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Xml.Linq;

namespace CustomLocalization
{
    /// <summary>
    /// Reads and writes to the app.xml configuration file.
    /// </summary>
    public static class ConfigurationManager
    {
        /// <summary>
        /// Contains the application settings.
        /// </summary>
        private static readonly IDictionary<string, string> AppSettings = new Dictionary<string, string>();

        /// <summary>
        /// The event that is set when the configuration data is finished loading.
        /// </summary>
        public static event EventHandler Loaded;

        /// <summary>
        /// The event fired when an error occurs retrieving configuration data.
        /// </summary>
        public static event EventHandler Error;

        /// <summary>
        /// True if the configuration data has been loaded, otherwise false. 
        /// </summary>
        public static bool IsLoaded;

        /// <summary>
        /// Loads the configuration settings into memory.
        /// </summary>
        public static void LoadSettings()
        {
            WebClient client = new WebClient();
            client.DownloadStringCompleted += ClientDownloadStringCompleted;
            client.DownloadStringAsync(new Uri("appSettings.xml", UriKind.Relative));
        }

        /// <summary>
        /// Retrieves an application setting.
        /// </summary>
        /// <param name="key">The key of the setting to retrieve.</param>
        /// <returns>The application setting.</returns>
        public static string GetSetting(string key)
        {
            if (!AppSettings.ContainsKey(key))
            {
                return string.Empty;
            }

            string setting = AppSettings[key];

            if (string.IsNullOrEmpty(setting))
            {
                return string.Empty;
            }

            if (setting.Contains("{"))
            {
                // Replace variables here.
            }

            return setting;
        }

        /// <summary>
        /// Handles the callback for downloading the app.xml file.
        /// </summary>
        /// <param name="sender">The source.</param>
        /// <param name="e">The arguments.</param>
        private static void ClientDownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                if (Error != null)
                {
                    Error(null, EventArgs.Empty);
                }

                return;
            }

            // load the data
            XDocument config;

            try
            {
                config = XDocument.Parse(e.Result);
            }
            catch (Exception)
            {
                if (Error != null)
                {
                    Error(null, EventArgs.Empty);
                }

                return;
            }

            var appsettings = from x in config.Descendants("AppSettings").Descendants("Add")
                              select new {Key = (string) x.Attribute("Key"), x.Attribute("Value").Value};

            // add to collection
            if (appsettings.Count() > 0)
            {
                foreach (var y in appsettings)
                {
                    AppSettings[y.Key] = y.Value;
                }
            }

            IsLoaded = true;

            if (Loaded != null)
            {
                Loaded(null, EventArgs.Empty);
            }
        }
    }
}