﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Net;
using System.Threading;
using System.Xml;

namespace CustomLocalization
{
    public class SmartResourceManager : INotifyPropertyChanged
    {
        private static readonly List<SmartResourceManager> Instances = new List<SmartResourceManager>();

        private static Dictionary<string, string> resourceSet;

        private static readonly Dictionary<string, Dictionary<string, string>> ResourceSets = new Dictionary<string, Dictionary<string, string>>();

        private readonly LocaleService.LocaleServiceClient localeClient = new LocaleService.LocaleServiceClient();

        public event EventHandler<CultureChangeErrorEventArgs> CultureChangeError;

        private static bool isInitialized;

        private static object lockObject = new object();

        public Dictionary<string, string> ResourceSet
        {
            get { return resourceSet; }
            set
            {
                resourceSet = value;

                // Notify all instances
                foreach (var obj in Instances)
                {
                    obj.NotifyPropertyChanged("ResourceSet");
                }
            }
        }

        public SmartResourceManager()
        {
            Instances.Add(this);
            localeClient.GetResourcesCompleted += LocaleClientGetResourcesCompleted;
        }

        public void Initialize(string neutralLanguageFile, string neutralLanguage)
        {
            lock (lockObject)
            {
                if (isInitialized) return;

                isInitialized = true;
            }

            if (string.IsNullOrWhiteSpace(neutralLanguageFile))
            {
                // No neutral resources
                ChangeCulture(Thread.CurrentThread.CurrentUICulture);
            }
            else
            {
                LoadNeutralResources(neutralLanguageFile, neutralLanguage);
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;


        private void LoadNeutralResources(string neutralLanguageFile, string neutralLanguage)
        {
            if (string.IsNullOrWhiteSpace(neutralLanguageFile)) return;

            try
            {
                WebClient client = new WebClient();
                client.DownloadStringCompleted += ClientDownloadStringCompleted;
                client.DownloadStringAsync(new Uri(neutralLanguageFile, UriKind.Relative), neutralLanguage);
            }
            catch (Exception)
            {
                if (!neutralLanguage.Equals(Thread.CurrentThread.CurrentUICulture.Name))
                    ChangeCulture(Thread.CurrentThread.CurrentUICulture);

                return;
            }
        }

        private void ClientDownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null && e.Result != null)
            {
                using (StringReader sr = new StringReader(e.Result))
                {
                    using (XmlReader xmlReader = XmlReader.Create(sr))
                    {
                        Dictionary<string, string> resources = ResxToDictionary(xmlReader);
                        ResourceSets.Add(e.UserState.ToString(), resources);
                        ResourceSet = resources;

                        // Now load the browsers active culture
                        if (!e.UserState.ToString().Equals(Thread.CurrentThread.CurrentUICulture.Name))
                            ChangeCulture(Thread.CurrentThread.CurrentUICulture);
                    }
                }
            }
        }
        
        private static Dictionary<string, string> ResxToDictionary(XmlReader resx)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            string keyname = string.Empty;

            while (resx.Read())
            {
                if (resx.IsStartElement() && resx.Name.Equals("value") && !string.IsNullOrWhiteSpace(keyname))
                {
                    string val = resx.ReadElementContentAsString();
                    result.Add(keyname, val);
                    keyname = string.Empty;
                }

                if (resx.IsStartElement() && resx.Name.Equals("data"))
                {
                    if (resx.GetAttribute("name") != null) keyname = resx.GetAttribute("name");
                }
            }
            return result;
        }

        private void LocaleClientGetResourcesCompleted(object sender, LocaleService.GetResourcesCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                var evt = CultureChangeError;

                if (evt != null)
                    evt(this, new CultureChangeErrorEventArgs(e.UserState as CultureInfo, e.Error));
            }
            else
            {
                if (e.Result == null) return;

                CultureInfo culture = e.UserState as CultureInfo;

                if (culture == null) return;

                ResourceSets.Add(culture.Name, e.Result);
                ResourceSet = e.Result;

                Thread.CurrentThread.CurrentCulture = Thread.CurrentThread.CurrentUICulture = culture;
            }
        }

        public void NotifyPropertyChanged(string property)
        {
                var evt = PropertyChanged;

                if (evt != null)
                {
                    evt(this, new PropertyChangedEventArgs(property));
                }
        }

        public void ChangeCulture(CultureInfo culture)
        {
            if (!ResourceSets.ContainsKey(culture.Name))
            {
                localeClient.GetResourcesAsync(culture.Name, culture);
            }
            else
            {
                ResourceSet = ResourceSets[culture.Name];

                Thread.CurrentThread.CurrentCulture = Thread.CurrentThread.CurrentUICulture = culture;
            }
        }

        public string GetString(string key)
        {
            if (string.IsNullOrEmpty(key)) return string.Empty;

            if (resourceSet.ContainsKey(key))
            {
                return resourceSet[key];
            }
            else
            {
                return string.Empty;
            }
        }

        public string GetString(string key, string culture)
        {
            if (ResourceSets.ContainsKey(culture))
            {
                if (ResourceSets[culture].ContainsKey(key))
                {
                    return ResourceSets[culture][key];
                }
                else
                {
                    return string.Empty;
                }
            }
            else
            {
                return string.Empty;
            }
        }
    }
}